/*
 Project name:
       LIN Hall Click
 Copyright:
      (c) mikroElektronika, 2014.
 Revision History:
      20140822:
      - Initial release (RD);
 Description:
      LIN HALL click is a simple solution for adding a linear Hall sensor to your design. 
      It carries an MLX90242 linear Hall sensor IC (with built-in active error correction circuitry) and an MCP3201 12-bit ADC. 
      LIN HALL click communicates with the target board through the mikroBUS SPI lines (CS, SCK, MISO). 
      The board is designed to use either a 3.3V or 5V power supply.
      LIN HALL click, with its linear Hall sensor, outputs a signal that is proportional to the flux density of a magnetic field. 
      The output signal will increase when a north magnetic field is applied, while a south magnetic field will cause it to decrease. 
      The ratiometric output can be used to measure the strength of different magnets, or to measure the distance between the sensor 
      and a magnet (which can be used for linear or rotary position sensing by placing magnets on the target object). LCD shows data.

 Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7 Connectivity
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8MHz Crystal, 32MHz MCU Clock
     ext. modules:    LIN Hall Click
                      http://www.mikroe.com/click/lin-hall/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 NOTES:
      - Place Lin Hall Click board in the mikroBUS socket 1.
      - Set development board J5 jumper according to SMD jumper on click board. By default it's soldered in the 3.3V position.
      - Turn On SW4-6 for LCD backlight.

*/

// HALL click module connections
sbit Chip_Select_Direction   at TRISE0_bit;
sbit Chip_Select             at LATE0_bit;
// eof HALL click module connections

// Lcd module connections
sbit LCD_RS at LATB4_bit;
sbit LCD_EN at LATB5_bit;
sbit LCD_D4 at LATB0_bit;
sbit LCD_D5 at LATB1_bit;
sbit LCD_D6 at LATB2_bit;
sbit LCD_D7 at LATB3_bit;

sbit LCD_RS_Direction at TRISB4_bit;
sbit LCD_EN_Direction at TRISB5_bit;
sbit LCD_D4_Direction at TRISB0_bit;
sbit LCD_D5_Direction at TRISB1_bit;
sbit LCD_D6_Direction at TRISB2_bit;
sbit LCD_D7_Direction at TRISB3_bit;
// End Lcd module connections

unsigned int measurement, lastValue;
char txt[10];


// Get ADC values
unsigned int getADC() {    // Returns 0..4095
  unsigned int tmp;
  Chip_Select = 0;                               // Select MCP3201
  tmp = SPI1_Read(0) & 0x1F;                     // Dummy 0 byte out , read in upper bits - B11-B7
  tmp = tmp << 7;                                // Shift ADC value by 7
  tmp += SPI1_Read(0) >> 1;                      // Dummy 0 byte out , read in lower  bits, B6-B0 + B1
                                                 // and shift right by 1 to determine B1
  Chip_Select= 1;                                // Deselect MCP3201
  return tmp;                                    // Returns 12-bit ADC value
}

void main() {
  ANSELB = 0;                      // Configure AN pins as digital
  ANSELC = 0;                      // Configure AN pins as digital
  ANSELE = 0;                      // Configure AN pins as digital
  SLRCON = 0;                      // Configure all PORTS at the standard Slew Rate

  Lcd_Init();                      // Initialize Lcd

  Lcd_Cmd(_LCD_CLEAR);             // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);        // Cursor off

  measurement = 0;                 // Initialize the measurement variable

  Chip_Select = 1;                 // Deselect MCP3201
  Chip_Select_Direction = 0;       // Set chip select pin to be output

  // Initialize SPI1 module at 500kHz, data sampled at the middle of interval
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
  SPI_Set_Active(&SPI1_Read, &SPI1_Write);      // Sets the SPI1 module active

  while (1) {
    measurement = getADC();       // Get ADC result
    WordToStr(measurement,txt);
    Lcd_Out(1,1,txt);
    Delay_ms(200);                // Wait 200ms
  }
  
}